/*GPL*START*
 * 
 * Copyright (C) 1998 by Johannes Overmann <overmann@iname.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * *GPL*END*/  

#ifndef _qjocakediagram_h_
#define _qjocakediagram_h_

#include <stdio.h>
#include <qwidget.h>
#include <qpixmap.h>
#include "tassocarray.h"

#define Qjo_cake_NUM_COLORS 10

class QjoCakeDiagram: public QWidget {

   Q_OBJECT;
   
 public:
   // cons & des
   QjoCakeDiagram(QWidget *parent);
   ~QjoCakeDiagram();
   
   // main interface
   void setSlice(const TString& name, double value);
   void setSlice(const TString& name, double value, const QColor& color);
   void clearSlices();
   QPixmap renderPixmap(int w, int h);
   
   // cutomization
   void setTitle(const char *title_) {title=title_;dirty=true;}
   void setFormat(const char *format_=0, const char *percent_format_=0, const char *value_format_=0);

 public slots:
   
 signals:
   void pressed(const char *name, const QPoint& p);
     
 protected:
   void paintEvent(QPaintEvent *p); 
   void resizeEvent(QResizeEvent *) {dirty=true;}
   void mousePressEvent(QMouseEvent *e);
   void mouseReleaseEvent(QMouseEvent *e);
   void mouseMoveEvent(QMouseEvent *e);
   
 private:
   // private Slice type
   struct Slice{
      TString name;
      double value;
      double percent;
      QColor color;      
   };

   // private methods
   void drawcake(QPixmap *p, int& cx, int& cy, int& rad);
   TString label(const Slice& sl) const;
   double argphi(int cx, int cy, int x, int y) const;
   double radius(int cx, int cy, int x, int y) const;

   // user data
   TAssocArray<TString, int> name2id;
   TArray<Slice> slice;
   TString title;
   TString format;
   TString percent_format;
   TString value_format;   
   QColor col_fg;
   QColor col_bg;
   QColor col_bor;
   QColor col_cbor;
   QColor col[Qjo_cake_NUM_COLORS];
   
   double rot;
   bool dirty;
   int cur_color;
   int next_id;
   QPixmap *pix;
   int cx;
   int cy;
   int rad;
   bool rotation;
   double oldrot;
};

#endif






